@@ -142,10 +142,12 @@ void pgp_fingerprints(void)
 		"one. See the manual for more information.\n"
 		"(Note: these fingerprints have nothing to do with SSH!)\n"
 		"\n"
-		"PuTTY Master Key (RSA), 1024-bit:\n"
-		"  " PGP_RSA_MASTER_KEY_FP "\n"
-		"PuTTY Master Key (DSA), 1024-bit:\n"
-		"  " PGP_DSA_MASTER_KEY_FP,
+                "PuTTY Master Key as of 2015 (RSA, 4096-bit):\n"
+                "  " PGP_MASTER_KEY_FP "\n\n"
+                "Original PuTTY Master Key (RSA, 1024-bit):\n"
+                "  " PGP_RSA_MASTER_KEY_FP "\n"
+                "Original PuTTY Master Key (DSA, 1024-bit):\n"
+                "  " PGP_DSA_MASTER_KEY_FP,
 		"PGP fingerprints", MB_ICONINFORMATION | MB_OK,
 		HELPCTXID(pgp_fingerprints));
 }
